
import pandas as pd
import matplotlib.pyplot as plt
import numpy as np
import scipy
import math
import statistics
import os
from sklearn.linear_model import LinearRegression
from scipy import stats

import statsmodels.api as sm
import statsmodels.formula.api as smf
from statsmodels.graphics.tsaplots import plot_acf
from statsmodels.graphics.tsaplots import plot_pacf

plt.rcParams['text.color'] = 'black'  # Set the text color to black
plt.rcParams['font.family'] = 'Times new roman'


os.getcwd()

#Data
data = pd.read_excel("main_data.xlsx")
data['date'] = pd.to_datetime(data['date'])

#Data manipulering
#data.iloc[871,4]=0.0039



data["incomeReturn"] = data["Carry"] + data["Rf"]
data["priceReturn"] = data["futucom"] - data["incomeReturn"] 

Indexes = data.copy()
Indexes.iloc[:,1:] = np.cumprod(Indexes.iloc[:, 1:]+1) 
data["tenyearreturn_futu"] = np.zeros(len(data))
data["tenyearreturn_spot"] = np.zeros(len(data))
data["tenyearincomereturn"] = np.zeros(len(data))
data["tenyearpriceReturn"] = np.zeros(len(data))


for i in range(len(data)-120+1):
    if i==0:
        data["tenyearreturn_futu"][i+120-1] = Indexes.futucom[i+120-1]-1
        data["tenyearreturn_spot"][i+120-1] = Indexes.spotcom[i+120-1]-1
        data["tenyearincomereturn"][i+120-1] = Indexes.incomeReturn[i+120-1]-1
        data["tenyearpriceReturn"][i+120-1] = Indexes.priceReturn[i+120-1]-1
    
        
    else:
        data["tenyearreturn_futu"][i+120-1] = Indexes.futucom[i+120-1]/Indexes.futucom[i-1]-1
        data["tenyearreturn_spot"][i+120-1] = Indexes.spotcom[i+120-1]/Indexes.spotcom[i-1]-1
        data["tenyearincomereturn"][i+120-1] = Indexes.incomeReturn[i+120-1]/Indexes.incomeReturn[i-1]-1
        data["tenyearpriceReturn"][i+120-1] = Indexes.priceReturn[i+120-1]/Indexes.priceReturn[i-1]-1




#Annulizing from ten year 

data["tenyearreturn_futu"] = (1+data["tenyearreturn_futu"])**(1/10)-1
data["tenyearreturn_spot"] = (1+data["tenyearreturn_spot"])**(1/10)-1
data["tenyearincomereturn"] = (1+data["tenyearincomereturn"])**(1/10)-1
data["tenyearpriceReturn"] = (1+data["tenyearpriceReturn"])**(1/10)-1



#
monthly_data = data.iloc[:, [0,5,6,13,14,15,20, 21]]
corr_monthly = monthly_data.corr()


###############
#10 year

tenyear_data = data.iloc[120-1:, [0,22, 23, 24, 25]]

corr_tenyear = tenyear_data.corr()

# figure 4.3

#Here comes the code for the graph of different coefs for income and price over different intervalles. 

interval = [1, 3, 6, 12, 24, 60, 120, 240]

price = np.zeros(len(interval))
income = np.zeros(len(interval))
t_income = np.zeros(len(interval))
t_price = np.zeros(len(interval))

data_coef = data.copy()

index = 0



for val in interval:
    data_coef["return_futu"] = np.zeros(len(data_coef))
    data_coef["return_spot"] = np.zeros(len(data_coef))
    data_coef["return_income"] = np.zeros(len(data_coef))
    data_coef["return_price"] = np.zeros(len(data_coef))
    print(val)


    for i in range(len(data_coef)-val+1):
        if i==0:
            data_coef["return_futu"][i+val-1] = Indexes.futucom[i+val-1]-1
            data_coef["return_spot"][i+val-1] = Indexes.spotcom[i+val-1]-1
            data_coef["return_income"][i+val-1] = Indexes.incomeReturn[i+val-1]-1
            data_coef["return_price"][i+val-1] = Indexes.priceReturn[i+val-1]-1
        
            
        else:
            data_coef["return_futu"][i+val-1] = Indexes.futucom[i+val-1]/Indexes.futucom[i-1]-1
            data_coef["return_spot"][i+val-1] = Indexes.spotcom[i+val-1]/Indexes.spotcom[i-1]-1
            data_coef["return_income"][i+val-1] = Indexes.incomeReturn[i+val-1]/Indexes.incomeReturn[i-1]-1
            data_coef["return_price"][i+val-1] = Indexes.priceReturn[i+val-1]/Indexes.priceReturn[i-1]-1

    #Bruk faktisk return som oppgitt, gjør om til annulized om det er mer enn 1 år med data.  
    if val >12:
        data_coef["return_futu"] = (1+data_coef["return_futu"])**(1/(val/12))-1
        data_coef["return_spot"] = (1+data_coef["return_spot"])**(1/(val/12))-1
        data_coef["return_income"] = (1+data_coef["return_income"])**(1/(val/12))-1
        data_coef["return_price"] = (1+data_coef["return_price"])**(1/(val/12))-1
        
        
    reg_Data = data_coef.iloc[val-1:, [0,26, 27, 28, 29]]

    print(reg_Data.corr())


    y = reg_Data.return_futu.to_numpy()
    x = reg_Data.return_income.to_numpy()
    x = sm.add_constant(x)

    #fit linear regression model
    model = sm.OLS(y, x).fit()
    if val >1 :
        model = model.get_robustcov_results(cov_type='HAC', maxlags=1, use_t=True)

    #view model summary
    print(model.summary())
    
    #Fyll inn riktig coef i henhold til 
    
    
    income[index] =  model.params[1]
    t_income[index] = model.tvalues[1]
    
    y = reg_Data.return_futu.to_numpy()
    x = reg_Data.return_spot.to_numpy()
    x = sm.add_constant(x)

    #fit linear regression model
    model = sm.OLS(y, x).fit()
    residuals = model.resid
    
    # Create PACF plot of the residuals
    fig, ax = plt.subplots(figsize=(10,5))
    plot_pacf(residuals, lags=8, ax=ax)
    plt.show()
    
    if val > 1:
        model = model.get_robustcov_results(cov_type='HAC', maxlags=1, use_t=True)

    #view model summary
    print(model.summary())
    
    
    
    price[index] = model.params[1]
    t_price[index] = model.tvalues[1]
    
    
    
    index += 1


#plot
plt.rcParams["figure.figsize"] = (12,6)

x_input = ["1 Month", "3 Month", "6 Month", "1 Year", "2 Years", "5 Years", "10 Years", "20 Years"]

# Assuming price, income, t_price and t_income are lists of the same length
plt.plot(x_input, price, "o-", color="#B22222", label = "Price coef")
plt.plot(x_input, income, "s-", color="#0072B2", label = "Income coef")

# Define offsets
offset_red = max(price) * 0.05  # 5% of the maximum price value
offset_blue = max(income) * 0.05  # 5% of the maximum income value

# Define custom y-positions for the red text for "10 Years" and "20 Years"
y_pos_red_10_years = price[x_input.index("10 Years")] + 0.012  # adjust as needed
y_pos_red_20_years = price[x_input.index("20 Years")] + 0.017  # adjust as needed

for i in range(len(x_input)):
    if x_input[i] == "10 Years":
        plt.text(x_input[i], y_pos_red_10_years, f'({t_price[i]:.2f})', va='bottom', ha='center', color='#B22222', fontsize = 12)
    elif x_input[i] == "20 Years":
        plt.text(x_input[i], y_pos_red_20_years, f'({t_price[i]:.2f})', va='bottom', ha='center', color='#B22222', fontsize = 12)
    else:
        plt.text(x_input[i], price[i] + offset_red, f'({t_price[i]:.2f})', va='bottom', ha='center', color='#B22222', fontsize = 12)
        
    if x_input[i] in ["1 Year", "2 Years", "5 Years", "10 Years", "20 Years"]:
        plt.text(x_input[i], income[i] - offset_blue, f'({t_income[i]:.2f})', va='top', ha='center', color='#0072B2', fontsize = 12)
    else:
        plt.text(x_input[i], income[i] + offset_blue, f'({t_income[i]:.2f})', va='bottom', ha='center', color='#0072B2', fontsize = 12)

plt.legend(fontsize = 16, loc = "lower right")
plt.grid()
plt.yticks(fontsize = 16)
plt.xticks(fontsize = 16)
plt.xlabel("Overlapping interval", fontsize = 16)
plt.ylabel("Coefficient", fontsize = 16)
plt.title("OLS-coefficients Explaining Total Return", fontsize = 26)
plt.show()