
import datetime
import pandas as pd
import matplotlib.pyplot as plt
import numpy as np
from scipy import stats as sc
import math
from tabulate import tabulate
from sklearn.linear_model import LinearRegression
from sklearn.metrics import mean_squared_error, r2_score
import statsmodels.api as sm
from scipy.stats.stats import pearsonr
import seaborn as sns
import seaborn as sns
sns.__version__
import os

os.getcwd()

#Data
data = pd.read_excel("main_data.xlsx")
data['date'] = pd.to_datetime(data['date'])
data["date"] = data["date"].astype("datetime64[ns]")
data["date"] = pd.to_datetime(data["date"]).dt.date
data.set_index("date", inplace=True)

data["60/40"] = 0.6 * data["exequity"] + 0.4 * data["exbond"]

data_pre1945 = data.iloc[0:826,:]
data_post1945 = data.iloc[826:,:]


#statistics for the different "states" (full sample) : 
stats = data.describe()
stats.loc["mean"] = (stats.loc["mean"])*12
stats.loc["std"] = stats.loc["std"]*np.sqrt(12)

# Descriptive stats for con/backwardation
contango_data = data[data.Stateofbackcon == 1] #contango = 1 
backwardation_data = data[data.Stateofbackcon == 0]
stats_con = contango_data.describe()
stats_con.loc["mean"] = (stats_con.loc["mean"])*12
stats_con.loc["std"] = stats_con.loc["std"]*np.sqrt(12)
stats_back = backwardation_data.describe()
stats_back.loc["mean"] = (stats_back.loc["mean"])*12
stats_back.loc["std"] = stats_back.loc["std"]*np.sqrt(12)

# Descriptive stats for UNEXPECTED inflation up/down
infup_data = data[data.statinf == 1] 
infdown_data = data[data.statinf == 0]
stats_infup = infup_data.describe()
stats_infup.loc["mean"] = (stats_infup.loc["mean"])*12
stats_infup.loc["std"] = stats_infup.loc["std"]*np.sqrt(12)

stats_infdown = infdown_data.describe()
stats_infdown.loc["mean"] = (stats_infdown.loc["mean"])*12
stats_infdown.loc["std"] = stats_infdown.loc["std"]*np.sqrt(12)

# Correlation matrix for different states/samples
corr_data = data.corr()
corr_data_pre45 = data_pre1945.corr()
corr_data_post45 = data_post1945.corr()

corr_infup = infup_data.corr()
corr_infdown = infdown_data.corr()

corr_back = backwardation_data.corr()
corr_con  = contango_data.corr()


# stats for different states
tab = np.matrix(data = np.zeros((13,5)))
#count fra states
tab[0,0] = stats.exfutucom[0]
tab[0,1] = stats_back.exfutucom[0]
tab[0,2] = stats_con.exfutucom[0]
tab[0,3] = stats_infup.exfutucom[0]
tab[0,4] = stats_infdown.exfutucom[0]

#60/40 mean ret fra states
tab[1,0] = stats["60/40"][1]
tab[1,1] = stats_back["60/40"][1]
tab[1,2] = stats_con["60/40"][1]
tab[1,3] = stats_infup["60/40"][1]
tab[1,4] = stats_infdown["60/40"][1]

#com mean fra  states
tab[2,0] = stats.exfutucom[1]
tab[2,1] = stats_back.exfutucom[1]
tab[2,2] = stats_con.exfutucom[1]
tab[2,3] = stats_infup.exfutucom[1]
tab[2,4] = stats_infdown.exfutucom[1]

#spot mean fra  states
tab[3,0] = stats.exspotcom[1]
tab[3,1] = stats_back.exspotcom[1]
tab[3,2] = stats_con.exspotcom[1]
tab[3,3] = stats_infup.exspotcom[1]
tab[3,4] = stats_infdown.exspotcom[1]

#Ira carry mean ret fra states
tab[4,0] = stats.iracarry[1]
tab[4,1] = stats_back.iracarry[1]
tab[4,2] = stats_con.iracarry[1]
tab[4,3] = stats_infup.iracarry[1]
tab[4,4] = stats_infdown.iracarry[1]

# Bond mean ret fra states
tab[5,0] = stats.exbond[1]
tab[5,1] = stats_back.exbond[1]
tab[5,2] = stats_con.exbond[1]
tab[5,3] = stats_infup.exbond[1]
tab[5,4] = stats_infdown.exbond[1]

#stock mean ret 
tab[6,0] = stats.exequity[1]
tab[6,1] = stats_back.exequity[1]
tab[6,2] = stats_con.exequity[1]
tab[6,3] = stats_infup.exequity[1]
tab[6,4] = stats_infdown.exequity[1]

# commodity volatility fra states 
tab[7,0] = stats.exfutucom[2]
tab[7,1] = stats_back.exfutucom[2]
tab[7,2] = stats_con.exfutucom[2]
tab[7,3] = stats_infup.exfutucom[2]
tab[7,4] = stats_infdown.exfutucom[2]

# commodity volatility fra states 
tab[8,0] = stats["60/40"][2]
tab[8,1] = stats_back["60/40"][2]
tab[8,2] = stats_con["60/40"][2]
tab[8,3] = stats_infup["60/40"][2]
tab[8,4] = stats_infdown["60/40"][2]


# Correlation with bonds
tab[9,0] = corr_data.exfutucom[3]
tab[9,1] = corr_back.exfutucom[3]
tab[9,2] = corr_con.exfutucom[3]
tab[9,3] = corr_infup.exfutucom[3]
tab[9,4] = corr_infdown.exfutucom[3]

# Correlation with stock 
tab[10,0] = corr_data.exfutucom[2]
tab[10,1] = corr_back.exfutucom[2]
tab[10,2] = corr_con.exfutucom[2]
tab[10,3] = corr_infup.exfutucom[2]
tab[10,4] = corr_infdown.exfutucom[2]


# Correlation between stocks and bonds 
tab[11,0] = corr_data.exequity[3]
tab[11,1] = corr_back.exequity[3]
tab[11,2] = corr_con.exequity[3]
tab[11,3] = corr_infup.exequity[3]
tab[11,4] = corr_infdown.exequity[3]


# Correlation between 6040 and commodity futures
tab[12,0] = corr_data.exfutucom[19]
tab[12,1] = corr_back.exfutucom[19]
tab[12,2] = corr_con.exfutucom[19]
tab[12,3] = corr_infup.exfutucom[19]
tab[12,4] = corr_infdown.exfutucom[19]


tab = pd.DataFrame(data = tab, columns = ["Full Sample", "Backwardation", "Contango", "Inflation Up", "Inflation Down"]) 
tab.rename({0:"#months",1:"60/40 portfolio", 2:"Commodity futures", 3:"Commdity spot", 4:"Interest adj. carry", 5:"Bonds", 6:"Stocks", 7:"Commodity futures volatility", 8:"60/40 portfolio volatility", 9:"Commodities vs Bonds", 10:"Commodities vs Stocks", 11:"Stocks vs Bonds", 12:"60/40 vs Commodities"}, inplace = True)

# endre til %
tab.loc[["Commodities vs Bonds", "Commodities vs Stocks", "Stocks vs Bonds", "60/40 vs Commodities"]] = tab.loc[["Commodities vs Bonds", "Commodities vs Stocks", "Stocks vs Bonds", "60/40 vs Commodities"]].round(2)
tab.loc[[ "60/40 portfolio","Commodity futures", "Commdity spot", "Interest adj. carry", "Bonds", "Stocks", "Commodity futures volatility", "60/40 portfolio volatility"]] = tab.loc[[ "60/40 portfolio","Commodity futures", "Commdity spot", "Interest adj. carry", "Bonds", "Stocks", "Commodity futures volatility", "60/40 portfolio volatility"]].mul(100).round(1).astype(str).add("%")

tab = tab.reset_index()

print(tab.to_latex(index = False, multirow = True))


#Tstats diff in mean between different states 

##full sample
tstat = np.zeros((7,3))
#commodity futures 
tstat[0,0] = sc.stats.ttest_ind(contango_data["exfutucom"], backwardation_data["exfutucom"])[0]
tstat[0,1] = sc.stats.ttest_ind(data_post1945["exfutucom"], data_pre1945["exfutucom"])[0]
tstat[0,2] = sc.stats.ttest_ind(infup_data["exfutucom"], infdown_data["exfutucom"])[0]

# spot 
tstat[1,0] = sc.stats.ttest_ind(contango_data["exspotcom"], backwardation_data["exspotcom"])[0]
tstat[1,1] = sc.stats.ttest_ind(data_post1945["exspotcom"], data_pre1945["exspotcom"])[0]
tstat[1,2] = sc.stats.ttest_ind(infup_data["exspotcom"], infdown_data["exspotcom"])[0]

# bond 
tstat[2,0] = sc.stats.ttest_ind(contango_data["exbond"], backwardation_data["exbond"])[0]
tstat[2,1] = sc.stats.ttest_ind(data_post1945["exbond"], data_pre1945["exbond"])[0]
tstat[2,2] = sc.stats.ttest_ind(infup_data["exbond"], infdown_data["exbond"])[0]

# equities 
tstat[3,0] = sc.stats.ttest_ind(contango_data["exequity"], backwardation_data["exequity"])[0]
tstat[3,1] = sc.stats.ttest_ind(data_post1945["exequity"], data_pre1945["exequity"])[0]
tstat[3,2] = sc.stats.ttest_ind(infup_data["exequity"], infdown_data["exequity"])[0]

# Interest adjusted carry 
tstat[4,0] = sc.stats.ttest_ind(contango_data["iracarry"], backwardation_data["iracarry"])[0]
tstat[4,1] = sc.stats.ttest_ind(data_post1945["iracarry"], data_pre1945["iracarry"])[0]
tstat[4,2] = sc.stats.ttest_ind(infup_data["iracarry"], infdown_data["iracarry"])[0]

# 60/40
tstat[5,0] = sc.stats.ttest_ind(contango_data["60/40"], backwardation_data["60/40"])[0]
tstat[5,1] = sc.stats.ttest_ind(data_post1945["60/40"], data_pre1945["60/40"])[0]
tstat[5,2] = sc.stats.ttest_ind(infup_data["60/40"], infdown_data["60/40"])[0]

# interest rate 
tstat[6,0] = sc.stats.ttest_ind(contango_data["Rf"], backwardation_data["Rf"])[0]
tstat[6,1] = sc.stats.ttest_ind(data_post1945["Rf"], data_pre1945["Rf"])[0]
tstat[6,2] = sc.stats.ttest_ind(infup_data["Rf"], infdown_data["Rf"])[0]

t_stats = pd.DataFrame(data= abs(tstat), index = ["Commodity return", "Excess spot return","Bond","Stock", "Interest rate-adjusted carry", "60/40", "Interest Rate"], columns = ["Backwardation/Contango",  "pre/post 1945","Inf up/down"])
t_stats = t_stats.round(2)
t_stats = t_stats.reset_index()
print(t_stats.to_latex(index = False, multirow = True))
