
import datetime
import pandas as pd
import matplotlib.pyplot as plt
import numpy as np
from scipy import stats as sc
import math
from tabulate import tabulate
from sklearn.linear_model import LinearRegression
from sklearn.metrics import mean_squared_error, r2_score
import statsmodels.api as sm
from scipy.stats.stats import pearsonr
import seaborn as sns
import seaborn as sns
sns.__version__


#Data
data = pd.read_excel("main_data.xlsx")
data['date'] = pd.to_datetime(data['date'])
data["date"] = data["date"].astype("datetime64[ns]")
data["date"] = pd.to_datetime(data["date"]).dt.date

data.set_index("date", inplace=True)

data["60/40"] = 0.6 * data["exequity"] + 0.4 * data["exbond"]

log_data = np.log(data+1)

# Calculate the rolling 12-month return for each month
y_rolling_ret = data.rolling(window=12).apply(lambda x: (1+x).prod()-1, raw=True).dropna()
q_rolling_ret =  data.rolling(window=3).apply(lambda x: (1+x).prod()-1, raw=True).dropna()
fiveyr_rolling_ret =  (data.rolling(window=60).apply(lambda x: (1+x).prod()-1, raw=True)).dropna()
tenyr_rolling_ret =  (data.rolling(window=120).apply(lambda x: (1+x).prod()-1, raw=True)).dropna()
twentyyr_rolling_ret =  (data.rolling(window=240).apply(lambda x: (1+x).prod()-1, raw=True)).dropna()



corr_with_inflation = np.zeros((6,4))
### Correlation of assets with inflation ref. table 5 facts and fantasies 
corr_inf = data[["exequity", "exbond", "exfutucom", "infMonth"]].corr() #same sign as paper
# monthly data
corr_with_inflation[0,0] = pearsonr(data["exequity"], data["infMonth"])[0] #returnerer corr og pvalue
corr_with_inflation[0,1] =pearsonr(data["exbond"], data["infMonth"])[0]    #eq, bond og futcom corr mot inf er signifikant
corr_with_inflation[0,2] =pearsonr(data["exfutucom"], data["infMonth"])[0]
corr_with_inflation[0,3] =pearsonr(data["60/40"], data["infMonth"])[0]
#gjør det samme 5 årlig, årlig og kvartalsvis 

# rolling data quarterly
corr_with_inflation[1,0] =pearsonr(q_rolling_ret["exequity"], q_rolling_ret["infMonth"])[0] #returnerer corr og pvalue
corr_with_inflation[1,1] =pearsonr(q_rolling_ret["exbond"], q_rolling_ret["infMonth"]) [0]    
corr_with_inflation[1,2] =pearsonr(q_rolling_ret["exfutucom"], q_rolling_ret["infMonth"])[0]
corr_with_inflation[1,3] =pearsonr(q_rolling_ret["60/40"], q_rolling_ret["infMonth"])[0]


# rolling data yearly
corr_with_inflation[2,0] =pearsonr(y_rolling_ret["exequity"], y_rolling_ret["infMonth"])[0] #returnerer corr og pvalue
corr_with_inflation[2,1] =pearsonr(y_rolling_ret["exbond"], y_rolling_ret["infMonth"])[0]     
corr_with_inflation[2,2] =pearsonr(y_rolling_ret["exfutucom"], y_rolling_ret["infMonth"])[0]
corr_with_inflation[2,3] =pearsonr(y_rolling_ret["60/40"], y_rolling_ret["infMonth"])[0]



# rolling data 5yr
corr_with_inflation[3,0] =pearsonr(fiveyr_rolling_ret["exequity"], fiveyr_rolling_ret["infMonth"])[0] #returnerer corr og pvalue
corr_with_inflation[3,1] =pearsonr(fiveyr_rolling_ret["exbond"], fiveyr_rolling_ret["infMonth"])[0]     
corr_with_inflation[3,2] =pearsonr(fiveyr_rolling_ret["exfutucom"], fiveyr_rolling_ret["infMonth"])[0]
corr_with_inflation[3,3] =pearsonr(fiveyr_rolling_ret["60/40"], fiveyr_rolling_ret["infMonth"])[0]

# rolling data 10yr
corr_with_inflation[4,0] =pearsonr(tenyr_rolling_ret["exequity"], tenyr_rolling_ret["infMonth"])[0] #returnerer corr og pvalue
corr_with_inflation[4,1] =pearsonr(tenyr_rolling_ret["exbond"], tenyr_rolling_ret["infMonth"])[0]     
corr_with_inflation[4,2] =pearsonr(tenyr_rolling_ret["exfutucom"], tenyr_rolling_ret["infMonth"])[0]
corr_with_inflation[4,3] =pearsonr(tenyr_rolling_ret["60/40"], tenyr_rolling_ret["infMonth"])[0]

# rolling data 20yr
corr_with_inflation[5,0] =pearsonr(twentyyr_rolling_ret["exequity"], twentyyr_rolling_ret["infMonth"])[0] #returnerer corr og pvalue
corr_with_inflation[5,1] =pearsonr(twentyyr_rolling_ret["exbond"], twentyyr_rolling_ret["infMonth"])[0]     
corr_with_inflation[5,2] =pearsonr(twentyyr_rolling_ret["exfutucom"], twentyyr_rolling_ret["infMonth"])[0]
corr_with_inflation[5,3] =pearsonr(twentyyr_rolling_ret["60/40"], twentyyr_rolling_ret["infMonth"])[0]



Corr_with_inf = pd.DataFrame(data = corr_with_inflation, index=["Monthly", "Quarterly", "1-year", "5-year", "10-year", "20-year"], columns = ["Equities", "Bonds", "Commodity futures", "60/40"])
Corr_with_inf = Corr_with_inf.round(2)
# all correlations are significant (see pvalue in function)
Corr_with_inf = Corr_with_inf.reset_index()
print(Corr_with_inf.to_latex(index = False, multirow = True))


#correlations with commodity futures over different rolling horizions
corr_with_comfutu = np.zeros((6,4))
### Correlation of assets with inflation ref. table 5 facts and fantasies 
# monthly data
corr_with_comfutu[0,0] = pearsonr(data["exequity"], data["exfutucom"])[0] #returnerer corr og pvalue
corr_with_comfutu[0,1] =pearsonr(data["exbond"], data["exfutucom"])[0]    #eq, bond og futcom corr mot inf er signifikant
corr_with_comfutu[0,2] =pearsonr(data["exspotcom"], data["exfutucom"])[0]
corr_with_comfutu[0,3] =pearsonr(data["60/40"], data["exfutucom"])[0]

#gjør det samme 5 årlig, årlig og kvartalsvis 

# rolling data quarterly
corr_with_comfutu[1,0] =pearsonr(q_rolling_ret["exequity"], q_rolling_ret["exfutucom"])[0] #returnerer corr og pvalue
corr_with_comfutu[1,1] =pearsonr(q_rolling_ret["exbond"], q_rolling_ret["exfutucom"]) [0]    
corr_with_comfutu[1,2] =pearsonr(q_rolling_ret["exspotcom"], q_rolling_ret["exfutucom"])[0]
corr_with_comfutu[1,3] =pearsonr(q_rolling_ret["60/40"], q_rolling_ret["exfutucom"])[0]


# rolling data yearly
corr_with_comfutu[2,0] =pearsonr(y_rolling_ret["exequity"], y_rolling_ret["exfutucom"])[0] #returnerer corr og pvalue
corr_with_comfutu[2,1] =pearsonr(y_rolling_ret["exbond"], y_rolling_ret["exfutucom"])[0]     
corr_with_comfutu[2,2] =pearsonr(y_rolling_ret["exspotcom"], y_rolling_ret["exfutucom"])[0]
corr_with_comfutu[2,3] =pearsonr(y_rolling_ret["60/40"], y_rolling_ret["exfutucom"])[0]


# rolling data 5yr
corr_with_comfutu[3,0] =pearsonr(fiveyr_rolling_ret["exequity"], fiveyr_rolling_ret["exfutucom"])[0] #returnerer corr og pvalue
corr_with_comfutu[3,1] =pearsonr(fiveyr_rolling_ret["exbond"], fiveyr_rolling_ret["exfutucom"])[0]     
corr_with_comfutu[3,2] =pearsonr(fiveyr_rolling_ret["exspotcom"], fiveyr_rolling_ret["exfutucom"])[0]
corr_with_comfutu[3,3] =pearsonr(fiveyr_rolling_ret["60/40"], fiveyr_rolling_ret["exfutucom"])[0]


# rolling data 10yr
corr_with_comfutu[4,0] =pearsonr(tenyr_rolling_ret["exequity"], tenyr_rolling_ret["exfutucom"])[0] #returnerer corr og pvalue
corr_with_comfutu[4,1] =pearsonr(tenyr_rolling_ret["exbond"], tenyr_rolling_ret["exfutucom"])[0]     
corr_with_comfutu[4,2] =pearsonr(tenyr_rolling_ret["exspotcom"], tenyr_rolling_ret["exfutucom"])[0]
corr_with_comfutu[4,3] =pearsonr(tenyr_rolling_ret["60/40"], tenyr_rolling_ret["exfutucom"])[0]


# rolling data 20tenyr_rolling_retyr
corr_with_comfutu[5,0] =pearsonr(twentyyr_rolling_ret["exequity"], twentyyr_rolling_ret["exfutucom"])[0] #returnerer corr og pvalue
corr_with_comfutu[5,1] =pearsonr(twentyyr_rolling_ret["exbond"], twentyyr_rolling_ret["exfutucom"])[0]     
corr_with_comfutu[5,2] =pearsonr(twentyyr_rolling_ret["exspotcom"], twentyyr_rolling_ret["exfutucom"])[0]
corr_with_comfutu[5,3] =pearsonr(twentyyr_rolling_ret["60/40"], twentyyr_rolling_ret["exfutucom"])[0]


Corr_with_comfutu = pd.DataFrame(data = corr_with_comfutu, index=["Monthly", "Quarterly", "1-year", "5-year", "10-year", "20-year"], columns = ["Equities", "Bonds", "Commodity spot", "60/40"])
Corr_with_comfutu = Corr_with_comfutu.round(2)
Corr_with_comfutu = Corr_with_comfutu.reset_index()
print(Corr_with_comfutu.to_latex(index = False, multirow = True))