import pandas as pd
import matplotlib.pyplot as plt
import numpy as np
import scipy
import math
import statistics
import os


os.getcwd()

#Data
data = pd.read_excel("main_data.xlsx")
data['date'] = pd.to_datetime(data['date'])

#Data manipulering
#data.iloc[871,4]=0.0039

#stats for excess returns of spot, futu, bond, equtiy, rf and 60/40
exReturns = data.iloc[:,[0,1,2,3,4]]
exReturns["60/40"] = 0.6 * exReturns["exequity"] + 0.4 * exReturns["exbond"]
exReturns["60/40_48Futu"] = 0.52 * exReturns["60/40"] + 0.48 * exReturns["exfutucom"]
exReturns["60/40_235Futu"] = 0.765 * exReturns["60/40"] + 0.235 * exReturns["exfutucom"]


exReturns = exReturns.join(data["Rf"])

exStats = exReturns.describe()
exStats.loc["mean"] = exStats.loc["mean"]*12
exStats.loc["std"] = exStats.loc["std"]*np.sqrt(12)
exStats.loc["SR"] = exStats.iloc[1]/exStats.iloc[2]

exStats.loc["skew"] = exReturns.skew()
exStats.loc["excessKurtosis"] = exReturns.kurtosis()


############
#stats for LNexcess returns of spot, futu, bond, equtiy, rf and 60/40

#Still normal returns, building the portfolio before transforming to LN returns. 
LNexReturns = data.iloc[:,[0,1,2,3,4]]
LNexReturns["60/40"] = 0.6 * LNexReturns["exequity"] + 0.4 * LNexReturns["exbond"]
LNexReturns["60/40_48Futu"] = 0.52 * LNexReturns["60/40"] + 0.48 * LNexReturns["exfutucom"]
LNexReturns["60/40_235Futu"] = 0.765 * LNexReturns["60/40"] + 0.235 * LNexReturns["exfutucom"]

LNexReturns = LNexReturns.join(data["Rf"])

#Going from normal returns to LN returns 
LNexReturns.iloc[:, 1:] = np.log(LNexReturns.iloc[:, 1:]+1)

LNexStats = LNexReturns.describe()
LNexStats.loc["mean"] = LNexStats.loc["mean"]*12
LNexStats.loc["std"] = LNexStats.loc["std"]*np.sqrt(12)
LNexStats.loc["SR"] = LNexStats.iloc[1]/LNexStats.iloc[2]

LNexStats.loc["skew"] = LNexReturns.skew()
LNexStats.loc["excessKurtosis"] = LNexReturns.kurtosis()


############################################################################
#Indekser:


Indexes = exReturns.copy()
Indexes.iloc[:,1:] = np.cumprod(Indexes.iloc[:, 1:]+1)


logIndexes = LNexReturns.copy()
logIndexes.iloc[:,1:] = np.cumsum(logIndexes.iloc[:, 1:])



###############################################################################
############################### Utility for mean variance no short and no leverage, no rf, and for full scale
Utility = pd.DataFrame()
Utility["Percentage"] = np.zeros(201)
Utility["Utility_mean"] = np.zeros(201)
Utility["Utility_fullscale"] = np.zeros(201)

theta = -0.03
v = 3

start_int = 0
increase = 0.005

test = pd.DataFrame()

for i in range(len(Utility)):
    weight = start_int + i * increase
    port = pd.DataFrame()
    port["port"] = exReturns["60/40"]*(1-weight) + exReturns["exfutucom"]*weight
    port["log"] = np.log(1+port["port"])
    Utility["Percentage"][i] = weight
    Utility["Utility_mean"][i] = port["log"].sum()
    
    port["fullscale"] = port["log"]
    for j in range(len(exReturns)):
        if port["port"][j] < theta:
            port["fullscale"][j] = v*(port["port"][j] - theta) + np.log(1+theta) 
            
    Utility["Utility_fullscale"][i] = port["fullscale"].sum()
    
    if(weight == 0.20):
        test = port.copy()

# By checking Utility we would see that the best for Utility full-scale is 23.5 % 
# Max wealth would be 48.5 %. commodity futures. 


######################
#Just testing for Commodity futures, bonds and equities without the restriction 
# of holding only 60/40 and commodity futures.
#If so, the optimal wealth would be 16 % futures, 84 % equity and 0 % bonds. :6.720821714390329
# Best full-scale is 38% futures, 62% equity and 0 % bonds. LN wealth: 6.577340403105867
#Interesting that it will not include Bonds. But you potensially do so if changing the params.

n = 51

Utility_all_mean = pd.DataFrame()
Utility_all_mean["Commodity"] = np.zeros(n)
Utility_all_mean["Equity"] = np.zeros(n)
Utility_all_mean["Bonds"] = np.zeros(n)
Utility_all_mean["Utility_all_mean"] = np.zeros(n)
Utility_all_mean["Utility_all_fullscale"] = np.zeros(n)

Utility_all_full = pd.DataFrame()
Utility_all_full["Commodity"] = np.zeros(n)
Utility_all_full["Equity"] = np.zeros(n)
Utility_all_full["Bonds"] = np.zeros(n)
Utility_all_full["Utility_all_mean"] = np.zeros(n)
Utility_all_full["Utility_all_fullscale"] = np.zeros(n)


port = pd.DataFrame()

theta = -0.03
v = 3

start_int_com = 0
start_int_bond = 0
increase = 0.02

for i in range(n):
    weight = start_int_com + i * increase
    ut = pd.DataFrame()
    ut["Commodity"] = np.zeros(n)
    ut["Equity"] = np.zeros(n)
    ut["Bonds"] = np.zeros(n)
    ut["Utility_all_mean"] = np.zeros(n)
    ut["Utility_all_fullscale"] = np.zeros(n)
    
    for k in range(n):
        
        bonds = start_int_bond + k *increase
        port["port"] = exReturns["exequity"]*(1-weight-(1-weight)*bonds) + exReturns["exbond"]*(1-weight)*bonds + exReturns["exfutucom"]*weight
        port["log"] = np.log(1+port["port"])
        ut["Commodity"][k] = weight
        ut["Equity"][k] = (1-weight-(1-weight)*bonds)
        ut["Bonds"][k] = (1-weight)*bonds 
        
        
        ut["Utility_all_mean"][k] = port["log"].sum()
        
        port["fullscale"] = port["log"]
        for j in range(len(exReturns)):
            if port["port"][j] < theta:
                port["fullscale"][j] = v*(port["port"][j] - theta) + np.log(1+theta) 
                
        ut["Utility_all_fullscale"][k] = port["fullscale"].sum()
        
    Utility_all_mean.loc[i] = ut[ut["Utility_all_mean"] == max(ut["Utility_all_mean"])].iloc[0,:] 
    Utility_all_full.loc[i] = ut[ut["Utility_all_fullscale"] == max(ut["Utility_all_fullscale"])].iloc[0, :] 
        


#################
#Mean variance portfolio on excess returns to be used further
#48 % futures

gamma = 1
data["60/40"] = data["equity"] *0.60 + data["bond"] *0.40
covar = exReturns[["60/40",  "exfutucom"]]

covar = covar.cov()



covar_inv = np.linalg.inv(covar.iloc[:,:]) 

means = [exReturns["60/40"].mean(), exReturns["exfutucom"].mean()]

enMatrix = np.array([1,1])

lamda = (np.matmul(np.matmul(enMatrix,covar_inv),means) - gamma)/np.matmul(np.matmul(enMatrix,covar_inv),enMatrix)


MV_weight = 1/gamma * np.matmul(covar_inv, means-lamda)




downside = pd.DataFrame(columns=['5%', '10%', "Mean", "Std", "SR", "Total LN return"])

downside.loc["Full Scale"] = [str(round(100*(len(exReturns[exReturns["60/40_235Futu"] < -0.05])/len(exReturns)),2))+"%", str(round(100*len(exReturns[exReturns["60/40_235Futu"] < -0.10])/len(exReturns),2))+"%", str(round(100*exStats["60/40_235Futu"]["mean"],2))+"%", str(round(100*exStats["60/40_235Futu"]["std"],2))+"%", round(np.mean(exReturns["60/40_235Futu"])*np.sqrt(12)/np.std(exReturns["60/40_235Futu"]),3) , round(logIndexes["60/40_235Futu"][1750],2)]
downside.loc["Mean Variance"] =  [str(round(100*(len(exReturns[exReturns["60/40_48Futu"] < -0.05])/len(exReturns)),2))+"%", str(round(100*len(exReturns[exReturns["60/40_48Futu"] < -0.10])/len(exReturns),2))+"%", str(round(100*exStats["60/40_48Futu"]["mean"],2))+"%", str(round(100*exStats["60/40_48Futu"]["std"],2))+"%", round(np.mean(exReturns["60/40_48Futu"])*np.sqrt(12)/np.std(exReturns["60/40_48Futu"]),3) , round(logIndexes["60/40_48Futu"][1750],2)]
downside.loc["60/40"] = [str(round(100*(len(exReturns[exReturns["60/40"] < -0.05])/len(exReturns)),2))+"%", str(round(100*len(exReturns[exReturns["60/40"] < -0.10])/len(exReturns),2))+"%", str(round(100*exStats["60/40"]["mean"],2))+"%", str(round(100*exStats["60/40"]["std"],2))+"%", round(np.mean(exReturns["60/40"])*np.sqrt(12)/np.std(exReturns["60/40"]),3) , round(logIndexes["60/40"][1750],2)]


###################################
#Portfolio asymmetry for mean variance gamma 1 og fullscale theta -0,03 og v = 3

assymentry = pd.DataFrame()
assymentry = pd.DataFrame(columns=["Port corr",'DownSide', 'UpSide', "Difference"])

futuW = 0.235
equityW = (1-futuW)*0.6 
bondW = (1-futuW)*0.4

exReturns["assym"] = bondW * exReturns["exbond"] + futuW * exReturns["exfutucom"] + equityW * exReturns["exequity"]


port_assym_down = exReturns[exReturns["assym"] <0]
port_assym_up = exReturns[exReturns["assym"] >0]

corr_ass_down = port_assym_down[["exequity", "exbond" , "exfutucom"]]

corr_ass_down = corr_ass_down.corr() 

assym_down_eq = equityW * (corr_ass_down["exequity"]["exbond"]*bondW/(bondW+futuW) + corr_ass_down["exequity"]["exfutucom"]*futuW/(bondW+futuW))
assym_down_bo = bondW * (corr_ass_down["exequity"]["exbond"]*equityW/(equityW+futuW) + corr_ass_down["exbond"]["exfutucom"]*futuW/(equityW+futuW))
assym_down_fu = futuW * (corr_ass_down["exfutucom"]["exbond"]*bondW/(bondW+equityW) + corr_ass_down["exequity"]["exfutucom"]*equityW/(bondW+equityW))
assym_down = assym_down_eq + assym_down_bo + assym_down_fu

corr_ass_up = port_assym_up[["exequity", "exbond" , "exfutucom"]]

corr_ass_up = corr_ass_up.corr() 

assym_up_eq = equityW * (corr_ass_up["exequity"]["exbond"]*bondW/(bondW+futuW) + corr_ass_up["exequity"]["exfutucom"]*futuW/(bondW+futuW))
assym_up_bo = bondW * (corr_ass_up["exequity"]["exbond"]*equityW/(equityW+futuW) + corr_ass_up["exbond"]["exfutucom"]*futuW/(equityW+futuW))
assym_up_fu = futuW * (corr_ass_up["exfutucom"]["exbond"]*bondW/(bondW+equityW) + corr_ass_up["exequity"]["exfutucom"]*equityW/(bondW+equityW))
assym_up = assym_up_eq + assym_up_bo + assym_up_fu



corr_ass = exReturns[["exequity", "exbond" , "exfutucom"]]
corr_ass = corr_ass.corr()

assym_eq = equityW * (corr_ass["exequity"]["exbond"]*bondW/(bondW+futuW) + corr_ass["exequity"]["exfutucom"]*futuW/(bondW+futuW))
assym_bo = bondW * (corr_ass["exequity"]["exbond"]*equityW/(equityW+futuW) + corr_ass["exbond"]["exfutucom"]*futuW/(equityW+futuW))
assym_fu = futuW * (corr_ass["exfutucom"]["exbond"]*bondW/(bondW+equityW) + corr_ass["exequity"]["exfutucom"]*equityW/(bondW+equityW))
assym_full = assym_eq + assym_bo + assym_fu


assym = assym_down - assym_up


assymentry.loc["Full Scale"] = [round(assym_full,4),round(assym_down,4), round(assym_up,4), round(assym,4)] 

# same now for mean variance

futuW = 0.48
equityW = (1-futuW)*0.6 
bondW = (1-futuW)*0.4

exReturns["assym"] = bondW * exReturns["exbond"] + futuW * exReturns["exfutucom"] + equityW * exReturns["exequity"]


port_assym_down = exReturns[exReturns["assym"] <0]
port_assym_up = exReturns[exReturns["assym"] >0]

corr_ass_down = port_assym_down[["exequity", "exbond" , "exfutucom"]]

corr_ass_down = corr_ass_down.corr() 

assym_down_eq = equityW * (corr_ass_down["exequity"]["exbond"]*bondW/(bondW+futuW) + corr_ass_down["exequity"]["exfutucom"]*futuW/(bondW+futuW))
assym_down_bo = bondW * (corr_ass_down["exequity"]["exbond"]*equityW/(equityW+futuW) + corr_ass_down["exbond"]["exfutucom"]*futuW/(equityW+futuW))
assym_down_fu = futuW * (corr_ass_down["exfutucom"]["exbond"]*bondW/(bondW+equityW) + corr_ass_down["exequity"]["exfutucom"]*equityW/(bondW+equityW))
assym_down = assym_down_eq + assym_down_bo + assym_down_fu

corr_ass_up = port_assym_up[["exequity", "exbond" , "exfutucom"]]

corr_ass_up = corr_ass_up.corr() 

assym_up_eq = equityW * (corr_ass_up["exequity"]["exbond"]*bondW/(bondW+futuW) + corr_ass_up["exequity"]["exfutucom"]*futuW/(bondW+futuW))
assym_up_bo = bondW * (corr_ass_up["exequity"]["exbond"]*equityW/(equityW+futuW) + corr_ass_up["exbond"]["exfutucom"]*futuW/(equityW+futuW))
assym_up_fu = futuW * (corr_ass_up["exfutucom"]["exbond"]*bondW/(bondW+equityW) + corr_ass_up["exequity"]["exfutucom"]*equityW/(bondW+equityW))
assym_up = assym_up_eq + assym_up_bo + assym_up_fu



corr_ass = exReturns[["exequity", "exbond" , "exfutucom"]]
corr_ass = corr_ass.corr()

assym_eq = equityW * (corr_ass["exequity"]["exbond"]*bondW/(bondW+futuW) + corr_ass["exequity"]["exfutucom"]*futuW/(bondW+futuW))
assym_bo = bondW * (corr_ass["exequity"]["exbond"]*equityW/(equityW+futuW) + corr_ass["exbond"]["exfutucom"]*futuW/(equityW+futuW))
assym_fu = futuW * (corr_ass["exfutucom"]["exbond"]*bondW/(bondW+equityW) + corr_ass["exequity"]["exfutucom"]*equityW/(bondW+equityW))
assym_full = assym_eq + assym_bo + assym_fu


assym = assym_down - assym_up


assymentry.loc["Mean Variance"] = [round(assym_full,4),round(assym_down,4), round(assym_up,4), round(assym,4)] 



futuW = 0
equityW = (1-futuW)*0.60 
bondW = (1-futuW)*0.40

exReturns["assym"] = bondW * exReturns["exbond"] + futuW * exReturns["exfutucom"] + equityW * exReturns["exequity"]


port_assym_down = exReturns[exReturns["assym"] <0]
port_assym_up = exReturns[exReturns["assym"] >0]

corr_ass_down = port_assym_down[["exequity", "exbond" , "exfutucom"]]

corr_ass_down = corr_ass_down.corr() 

assym_down_eq = equityW * (corr_ass_down["exequity"]["exbond"]*bondW/(bondW+futuW) + corr_ass_down["exequity"]["exfutucom"]*futuW/(bondW+futuW))
assym_down_bo = bondW * (corr_ass_down["exequity"]["exbond"]*equityW/(equityW+futuW) + corr_ass_down["exbond"]["exfutucom"]*futuW/(equityW+futuW))
assym_down_fu = futuW * (corr_ass_down["exfutucom"]["exbond"]*bondW/(bondW+equityW) + corr_ass_down["exequity"]["exfutucom"]*equityW/(bondW+equityW))
assym_down = assym_down_eq + assym_down_bo + assym_down_fu

corr_ass_up = port_assym_up[["exequity", "exbond" , "exfutucom"]]

corr_ass_up = corr_ass_up.corr() 

assym_up_eq = equityW * (corr_ass_up["exequity"]["exbond"]*bondW/(bondW+futuW) + corr_ass_up["exequity"]["exfutucom"]*futuW/(bondW+futuW))
assym_up_bo = bondW * (corr_ass_up["exequity"]["exbond"]*equityW/(equityW+futuW) + corr_ass_up["exbond"]["exfutucom"]*futuW/(equityW+futuW))
assym_up_fu = futuW * (corr_ass_up["exfutucom"]["exbond"]*bondW/(bondW+equityW) + corr_ass_up["exequity"]["exfutucom"]*equityW/(bondW+equityW))
assym_up = assym_up_eq + assym_up_bo + assym_up_fu



corr_ass = exReturns[["exequity", "exbond" , "exfutucom"]]
corr_ass = corr_ass.corr()

assym_eq = equityW * (corr_ass["exequity"]["exbond"]*bondW/(bondW+futuW) + corr_ass["exequity"]["exfutucom"]*futuW/(bondW+futuW))
assym_bo = bondW * (corr_ass["exequity"]["exbond"]*equityW/(equityW+futuW) + corr_ass["exbond"]["exfutucom"]*futuW/(equityW+futuW))
assym_fu = futuW * (corr_ass["exfutucom"]["exbond"]*bondW/(bondW+equityW) + corr_ass["exequity"]["exfutucom"]*equityW/(bondW+equityW))
assym_full = assym_eq + assym_bo + assym_fu


assym = assym_down - assym_up


assymentry.loc["60/40"] = [round(assym_full,4),round(assym_down,4), round(assym_up,4), round(assym,4)] 


assymentry = assymentry.transpose()


#variance decomposition
asset1 = "exfutucom"
asset2 = "60/40"
port = "60/40_48Futu"

WA = 0.48
WB = 1-WA

varA = exReturns[asset1].var()
varB = exReturns[asset2].var()

covAB = np.cov(exReturns[asset1].values,exReturns[asset2].values)[1,0]


VarP_cal = WA**2 * varA + WB**2*varB + 2*WA*WB*covAB

varP = exReturns[port].var()



#### Part from commodity futures

part_of = (WA**2 * varA  +  WA*WB*covAB) / VarP_cal

part_of_2 = (WB**2 * varB  +  WA*WB*covAB) / VarP_cal



asset1 = "exfutucom"
asset2 = "60/40"
port = "60/40_235Futu"

WA = 0.235
WB = 1-WA

varA = exReturns[asset1].var()
varB = exReturns[asset2].var()

covAB = np.cov(exReturns[asset1].values,exReturns[asset2].values)[1,0]


VarP_cal = WA**2 * varA + WB**2*varB + 2*WA*WB*covAB

varP = exReturns[port].var()



#### Part from commodity futures

part_of = (WA**2 * varA  +  WA*WB*covAB) / VarP_cal

part_of_2 = (WB**2 * varB  +  WA*WB*covAB) / VarP_cal



































